const last = require('lodash/last')

const {
  services,
  Email,
  Fragment,
  Collection,
} = require('pubsweet-component-helper-service')

module.exports = models => async (req, res) => {
  const { collectionId, invitationId } = req.params
  const { invitationToken } = req.body

  if (!services.checkForUndefinedParams(invitationToken))
    return res.status(400).json({ error: 'Token is required' })

  const UserModel = models.User
  try {
    const user = await UserModel.findOneByField(
      'invitationToken',
      invitationToken,
    )
    const collection = await models.Collection.find(collectionId)
    const invitation = await collection.invitations.find(
      invitation => invitation.id === invitationId,
    )
    if (invitation === undefined)
      return res.status(404).json({
        error: `Invitation ${invitationId} not found`,
      })
    if (invitation.hasAnswer)
      return res
        .status(400)
        .json({ error: `Invitation has already been answered.` })
    if (invitation.userId !== user.id)
      return res.status(403).json({
        error: `User ${user.email} is not allowed to modify invitation ${
          invitation.id
        }`,
      })

    invitation.respondedOn = Date.now()
    invitation.hasAnswer = true
    invitation.isAccepted = false
    await collection.save()
    const collectionHelper = new Collection({ collection })
    const fragment = await models.Fragment.find(last(collection.fragments))
    const fragmentHelper = new Fragment({ fragment })
    const parsedFragment = await fragmentHelper.getFragmentData({
      handlingEditor: collection.handlingEditor,
    })
    const baseUrl = services.getBaseUrl(req)
    const {
      authorsList: authors,
      submittingAuthor,
    } = await collectionHelper.getAuthorData({ UserModel })
    const emailHelper = new Email({
      UserModel,
      collection,
      parsedFragment,
      baseUrl,
      authors,
    })
    emailHelper.setupReviewerDecisionEmail({
      authorName: `${submittingAuthor.firstName} ${submittingAuthor.lastName}`,
      agree: false,
      user,
    })
    return res.status(200).json({})
  } catch (e) {
    const notFoundError = await services.handleNotFoundError(e, 'item')
    return res.status(notFoundError.status).json({
      error: notFoundError.message,
    })
  }
}
