const config = require('config')
const logger = require('@pubsweet/logger')
const last = require('lodash/last')
const mailService = require('pubsweet-component-mail-service')

const {
  services,
  Team,
  Email,
  Fragment,
  Collection,
  authsome: authsomeHelper,
} = require('pubsweet-component-helper-service')

const statuses = config.get('statuses')
module.exports = models => async (req, res) => {
  const { collectionId, invitationId } = req.params
  const teamHelper = new Team({ TeamModel: models.Team, collectionId })

  try {
    const collection = await models.Collection.find(collectionId)
    const collectionHelper = new Collection({ collection })
    const authsome = authsomeHelper.getAuthsome(models)
    const target = {
      collection,
      path: req.route.path,
    }
    const canDelete = await authsome.can(req.user, 'DELETE', target)
    if (!canDelete)
      return res.status(403).json({
        error: 'Unauthorized.',
      })
    const invitation = await collection.invitations.find(
      invitation => invitation.id === invitationId,
    )
    if (invitation === undefined) {
      res.status(404).json({
        error: `Invitation ${invitationId} not found`,
      })
      return
    }

    const team = await teamHelper.getTeamByGroupAndCollection({
      role: invitation.role,
    })

    collection.invitations = collection.invitations.filter(
      inv => inv.id !== invitation.id,
    )
    if (invitation.role === 'handlingEditor') {
      collection.status = 'submitted'
      collection.visibleStatus = statuses[collection.status].private
      delete collection.handlingEditor
    } else if (invitation.role === 'reviewer') {
      await collectionHelper.updateStatusByNumberOfReviewers()
    }
    await collection.save()
    await teamHelper.removeTeamMember({
      teamId: team.id,
      userId: invitation.userId,
    })
    const UserModel = models.User
    const user = await UserModel.find(invitation.userId)
    user.teams = user.teams.filter(userTeamId => team.id !== userTeamId)
    await user.save()
    try {
      if (invitation.role === 'handlingEditor') {
        await mailService.sendSimpleEmail({
          toEmail: user.email,
          emailType: 'revoke-handling-editor',
        })
      } else if (invitation.role === 'reviewer') {
        const collectionHelper = new Collection({ collection })
        const fragment = await models.Fragment.find(last(collection.fragments))
        const fragmentHelper = new Fragment({ fragment })
        const parsedFragment = await fragmentHelper.getFragmentData({
          handlingEditor: collection.handlingEditor,
        })
        const baseUrl = services.getBaseUrl(req)
        const {
          authorsList: authors,
          submittingAuthor,
        } = await collectionHelper.getAuthorData({ UserModel })
        const emailHelper = new Email({
          UserModel,
          collection,
          parsedFragment,
          baseUrl,
          authors,
        })
        emailHelper.setupReviewerUnassignEmail({
          user,
          authorName: `${submittingAuthor.firstName} ${
            submittingAuthor.lastName
          }`,
        })
      }

      return res.status(200).json({})
    } catch (e) {
      logger.error(e.message)
      return res.status(500).json({ error: 'Email could not be sent.' })
    }
  } catch (e) {
    const notFoundError = await services.handleNotFoundError(e, 'collection')
    return res.status(notFoundError.status).json({
      error: notFoundError.message,
    })
  }
}
