const config = require('config')
const {
  services,
  Team,
  Invitation,
  authsome: authsomeHelper,
} = require('pubsweet-component-helper-service')

const configRoles = config.get('roles')

module.exports = models => async (req, res) => {
  const { role } = req.query
  if (!services.checkForUndefinedParams(role)) {
    res.status(400).json({ error: 'Role is required' })
    return
  }

  if (!configRoles.collection.includes(role)) {
    res.status(400).json({ error: `Role ${role} is invalid` })
    return
  }

  const { collectionId } = req.params
  const teamHelper = new Team({ TeamModel: models.Team, collectionId })

  try {
    const collection = await models.Collection.find(collectionId)
    const authsome = authsomeHelper.getAuthsome(models)
    const target = {
      collection,
      path: req.route.path,
    }
    const canGet = await authsome.can(req.user, 'GET', target)

    if (!canGet)
      return res.status(403).json({
        error: 'Unauthorized.',
      })

    const members = await teamHelper.getTeamMembersByCollection({
      role,
    })
    if (members === undefined) return res.status(200).json([])

    // TO DO: handle case for when the invitationID is provided
    const membersData = members.map(async member => {
      const user = await models.User.find(member)
      const invitationHelper = new Invitation({ userId: user.id, role })

      const {
        invitedOn,
        respondedOn,
        status,
        id,
      } = invitationHelper.getInvitationsData({
        invitations: collection.invitations,
      })

      return {
        name: `${user.firstName} ${user.lastName}`,
        invitedOn,
        respondedOn,
        email: user.email,
        status,
        userId: user.id,
        invitationId: id,
      }
    })

    const resBody = await Promise.all(membersData)
    res.status(200).json(resBody)
  } catch (e) {
    const notFoundError = await services.handleNotFoundError(e, 'collection')
    return res.status(notFoundError.status).json({
      error: notFoundError.message,
    })
  }
}
